#
# spec file
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor %{nil}
%if "%{flavor}" == "python"
%global pprefix python-
%define oldpython python
%bcond_without python
%bcond_without python2
%else
%global pprefix %{nil}
%bcond_with python
%bcond_with python2
%endif

%define with_fence_sanlockd 0
%define with_sanlk_reset    0
%if 0%{?suse_version} > 1320
%define with_fence_sanlockd 1
%define with_sanlk_reset    1
%endif
#Compat macro for new _fillupdir macro introduced in Nov 2017
%if ! %{defined _fillupdir}
  %define _fillupdir %{_localstatedir}/adm/fillup-templates
%endif
%define pname   sanlock
Name:           %{pprefix}%{pname}
Version:        3.9.5
Release:        1.2
%if ! %{with python}
Summary:        A shared disk lock manager
License:        GPL-2.0-only AND GPL-2.0-or-later AND LGPL-2.1-or-later
Group:          System/Base
%else
Summary:        Python bindings for the sanlock library
License:        GPL-2.0-only AND GPL-2.0-or-later AND LGPL-2.1-or-later
Group:          System/Base
%endif
URL:            https://pagure.io/sanlock
Source0:        %{pname}-%{version}.tar.xz
Source1:        sysconfig.sanlock
Source2:        sysconfig.wdmd
Source3:        fence_sanlockd.init
# Upstream patches
# SUSE patches
Patch100:       sanlock-SCHED_RESET_ON_FORK-undefined.patch
Patch101:       sanlock-python-prefix.patch
Patch102:       suse-systemd.patch
Patch103:       suse-no-date-time.patch
Patch104:       harden_fence_sanlockd.service.patch
Patch105:       harden_sanlk-resetd.service.patch
BuildRequires:  %{python_module devel}
BuildRequires:  %{python_module setuptools}
BuildRequires:  libaio-devel
BuildRequires:  pkgconfig
BuildRequires:  python-rpm-macros
BuildRequires:  systemd-rpm-macros
BuildRequires:  pkgconfig(blkid)
BuildRequires:  pkgconfig(uuid)
%if ! %{with python}
Requires(pre):  %fillup_prereq
Requires(pre):  shadow
Provides:       group(sanlock)
Provides:       user(sanlock)
Recommends:     logrotate
%{?systemd_requires}
%if 0%{?suse_version} >= 1500
Requires(pre):  group(disk)
%endif
%else
BuildRequires:  %{pname}-devel = %{version}
Provides:       sanlock-python
%if "%{python_flavor}" == "python2"
Provides:       %{oldpython}-%{pname}
%endif
%endif
%python_subpackages

%description
sanlock uses disk paxos to manage leases on shared storage.
Hosts connected to a common SAN can use this to synchronize their
access to the shared disks.
%if %{with python}
This package provides a module that permits applications written in
the Python programming language to use the interface supplied by the
sanlock library.
%endif

%package        -n libsanlock1
Summary:        A shared disk lock manager library
Group:          Development/Libraries/C and C++
Provides:       sanlock-lib = %{version}

%description    -n libsanlock1
The runtime libraries for sanlock, a shared disk lock manager.
Hosts connected to a common SAN can use this to synchronize their
access to the shared disks.

%package        -n %{pname}-devel
Summary:        Development files for %{pname}
Group:          Development/Libraries/C and C++
Requires:       libsanlock1 = %{version}-%{release}

%description    -n %{pname}-devel
The %{pname}-devel package contains libraries and header files for
developing applications that use %{pname}.

%package        -n fence-sanlock
Summary:        Fence agent using sanlock and wdmd
Group:          System/Base
Requires:       sanlock = %{version}-%{release}

%description    -n fence-sanlock
Fence agent and daemon for using sanlock and wdmd as a cluster fence agent.

%package -n     sanlk-reset
Summary:        Host reset daemon and client using sanlock
Group:          System/Base
Requires:       libsanlock1 = %{version}-%{release}
Requires:       sanlock = %{version}-%{release}

%description -n sanlk-reset
The reset daemon and client for sanlock.
A cooperating host running the daemon can be reset by a host
running the client, so long as both maintain access to a
common sanlock lockspace.

%prep
%setup -q -n %{pname}-%{version}
%patch -P 100
%patch -P 101
%patch -P 102 -p1
%patch -P 103 -p1
%patch -P 104 -p1
%patch -P 105 -p1

%build
%if ! %{with python}
# upstream does not require configure
# upstream does not support _smp_mflags
CFLAGS="%{optflags}" make -j1 -C wdmd
CFLAGS="%{optflags}" make -j1 -C src
%if %{with_fence_sanlockd}
CFLAGS="%{optflags}" make -j1 -C fence_sanlock
%endif
%if %{with_sanlk_reset}
CFLAGS="%{optflags}" make -j1 -C reset
%endif
%else
pushd python
CFLAGS="%{optflags} -fno-strict-aliasing" %python_build
popd
%endif

%install
%if ! %{with python}
%make_install LIBDIR=%{_libdir} -C src
%make_install LIBDIR=%{_libdir} -C wdmd
%if %{with_fence_sanlockd}
%make_install LIBDIR=%{_libdir} -C fence_sanlock
%endif
%if %{with_sanlk_reset}
%make_install LIBDIR=%{_libdir} -C reset
%endif

install -D -m 644 src/sanlock.conf %{buildroot}/%{_sysconfdir}/sanlock/sanlock.conf
install -D -m 644 %{SOURCE1} %{buildroot}/%{_fillupdir}/sysconfig.sanlock
install -D -m 644 %{SOURCE2} %{buildroot}/%{_fillupdir}/sysconfig.wdmd

install -D -m 644 init.d/sanlock.service %{buildroot}/%{_unitdir}/sanlock.service
ln -s service %{buildroot}%{_sbindir}/rcsanlock
install -D -m 644 init.d/wdmd.service %{buildroot}/%{_unitdir}/wdmd.service
ln -s service %{buildroot}%{_sbindir}/rcwdmd
%if %{with_fence_sanlockd}
install -D -m 0755 %{SOURCE3} %{buildroot}%{_prefix}/lib/systemd/systemd-fence_sanlockd
install -D -m 0644 init.d/fence_sanlockd.service %{buildroot}/%{_unitdir}/fence_sanlockd.service
ln -s service %{buildroot}%{_sbindir}/rcfence_sanlockd
%endif
%if %{with_sanlk_reset}
install -D -m 0644 init.d/sanlk-resetd.service %{buildroot}/%{_unitdir}/sanlk-resetd.service
ln -s service %{buildroot}%{_sbindir}/rcsanlk-resetd
%endif

install -Dm 0644 src/logrotate.sanlock \
	%{buildroot}%{_sysconfdir}/logrotate.d/sanlock

install -Dd -m 0755 %{buildroot}%{_sysconfdir}/wdmd.d
%else
pushd python
%python_install
popd
%endif

%if ! %{with python}
%pre -n %{pname}
getent group sanlock > /dev/null || groupadd \
	-g 179 sanlock
getent passwd sanlock > /dev/null || useradd \
	-u 179 -c "sanlock" -s /sbin/nologin -r \
	-g 179 -G disk -d %{_localstatedir}/run/sanlock sanlock

%service_add_pre wdmd.service
%service_add_pre sanlock.service

%pre -n fence-sanlock
%service_add_pre fence_sanlockd.service

%pre -n sanlk-reset
%service_add_pre sanlk-resetd.service

%post -n %{pname}
%service_add_post wdmd.service sanlock.service

%fillup_only -n wdmd
%fillup_only -n sanlock

%post -n libsanlock1 -p /sbin/ldconfig

%if %{with_fence_sanlockd}
%post -n fence-sanlock
%service_add_post fence_sanlockd.service
%endif

%post -n sanlk-reset
%service_add_post sanlk-resetd.service

%preun -n %{pname}
%service_del_preun wdmd.service sanlock.service

%preun -n fence-sanlock
%service_del_preun fence_sanlockd.service

%preun -n sanlk-reset
%service_del_preun sanlk-resetd.service

%postun -n %{pname}
%service_del_postun wdmd.service sanlock.service

%postun -n libsanlock1 -p /sbin/ldconfig

%postun -n fence-sanlock
%service_del_postun fence_sanlockd.service

%postun -n sanlk-reset
%service_del_postun sanlk-resetd.service

%files -n %{pname}
%dir %attr(0700, root, root) %{_sysconfdir}/wdmd.d/
%dir %attr(0700, root, root) %{_sysconfdir}/sanlock/
%config(noreplace) %{_sysconfdir}/sanlock/sanlock.conf
%{_sbindir}/rcsanlock
%{_fillupdir}/sysconfig.sanlock
%{_sbindir}/rcwdmd
%{_fillupdir}/sysconfig.wdmd
%{_unitdir}/sanlock.service
%{_unitdir}/wdmd.service
%{_sbindir}/sanlock
%{_sbindir}/wdmd
%{_mandir}/man8/wdmd*
%{_mandir}/man8/sanlock*
%config(noreplace) %{_sysconfdir}/logrotate.d/sanlock

%files -n libsanlock1
%{_libdir}/libsanlock.so.*
%{_libdir}/libsanlock_client.so.*
%{_libdir}/libwdmd.so.*

%files -n %{pname}-devel
%{_libdir}/libwdmd.so
%{_includedir}/wdmd.h
%{_libdir}/libsanlock.so
%{_libdir}/libsanlock_client.so
%{_includedir}/sanlock.h
%{_includedir}/sanlock_rv.h
%{_includedir}/sanlock_admin.h
%{_includedir}/sanlock_resource.h
%{_includedir}/sanlock_direct.h
%{_libdir}/pkgconfig/libsanlock.pc
%{_libdir}/pkgconfig/libsanlock_client.pc

%if %{with_fence_sanlockd}
%files -n fence-sanlock
%{_sbindir}/fence_sanlockd
%{_prefix}/lib/systemd/systemd-fence_sanlockd
%{_unitdir}/fence_sanlockd.service
%{_sbindir}/fence_sanlock
%{_sbindir}/fence_sanlockd
%{_sbindir}/rcfence_sanlockd
%{_mandir}/man8/fence_sanlock*
%endif

%if %{with_sanlk_reset}
%files -n sanlk-reset
%{_sbindir}/sanlk-reset
%{_sbindir}/sanlk-resetd
%{_sbindir}/rcsanlk-resetd
%{_unitdir}/sanlk-resetd.service
%{_mandir}/man8/sanlk-reset.8%{?ext_man}
%{_mandir}/man8/sanlk-resetd.8%{?ext_man}
%endif

%else

%files %{python_files}
%{python_sitearch}/sanlock*.so
%{python_sitearch}/sanlock_python-%{version}*info
%endif

%changelog
* Wed Oct 16 2024 jfehlig@suse.com
- Update to version 3.9.5:
  * sanlock: change log level for resource names
  * sanlock: include worker thread info in status
  * sanlock: reuse io buffer with hugepages
  * sanlock: use hugepages to avoid splitting read io
  * sanlock: don't reduce max_sectors_kb
  * sanlock: fix is dead log message for unused host id
  * sanlock: repeat helper_msg string termination
  * sanlock: fix invalid strcpy in direct dump
* Mon Jul  1 2024 James Fehlig <jfehlig@suse.com>
- suse-systemd.patch: Remove redundant 'ExecStart' from wdmd.service
  boo#1227208
* Mon Jun  3 2024 James Fehlig <jfehlig@suse.com>
- Update to sanlock 3.9.3
  - Make the watchdog timeout configurable
  - wdmd: close watchdog when not used
  - userstorage: replace by PyPI userstorage
  - sanlock: skip short delay in delta lease reacquire
  - sanlock: use product_uuid for host name
  - wdmd: fix timing for iTCO_wdt
  - sanlock: save client_id in token and report in status and use in -C
  - sanlock: include startup message in syslog
  - sanlock: improve repeated ballots in paxos_acquire
  - sanlock: fix zero io timeout for direct requests
  - sanlock: fix release writing zero dblock values
  - sanlock: skip delay when same host acquires delta lease
* Mon Feb 26 2024 Dominique Leuenberger <dimstar@opensuse.org>
- Use %%patch -P N instead of deprecated %%patchN.
* Mon Jan 29 2024 Dirk Müller <dmueller@suse.com>
- add user()/group() provides for rpm 4.19
* Tue Oct 18 2022 James Fehlig <jfehlig@suse.com>
- Update to sanlock 3.8.5
  - python: Replace distutils with setuptools
  - sanlock: fix memory leak of lockspace renewal_history
  - sanlock: fix pthread_create error check
  - sanlock: use helper to set max_sectors_kb
* Tue Nov 16 2021 Johannes Segitz <jsegitz@suse.com>
- Added hardening to systemd service(s) (bsc#1181400). Added patch(es):
  * harden_fence_sanlockd.service.patch
  * harden_sanlk-resetd.service.patch
* Wed Jun  9 2021 James Fehlig <jfehlig@suse.com>
- Update to sanlock 3.8.4
  - sanlock: socket send and recv retries
  - python: expose {get,set}_lvb
  - sanlock: Shrink thread pool when there is no work
  - config: Add max_worker_threads
* Fri Oct 23 2020 Benjamin Greiner <code@bnavigator.de>
-  Make python subpackage ready for multiple python3 flavors
  using the multibuild feature gh#openSUSE/python-rpm-macros#66
* Tue Aug 18 2020 James Fehlig <jfehlig@suse.com>
- Update to sanlock 3.8.2
  - Remove extra link args
  - Add write_init_io_timeout
  - Disable high_priority SCHED_RR by default
  - Dropped patches:
    sanlock-lto-disable-fpie.patch
- Add tar_scm source service and use xz compression
* Mon Jul 13 2020 James Fehlig <jfehlig@suse.com>
- Update to sanlock 3.8.1
  - python: add flake8 env
  - Add client debug logging
  _ Add per command debug logging
  - Misc bug fixes and improvements
* Fri Aug 16 2019 Dominique Leuenberger <dimstar@opensuse.org>
- Addpkgconfig(uuid) BuildRequires: so far we relied on util-linux'
  blkid to pull in uuid for us.
* Mon Jun 24 2019 Martin Liška <mliska@suse.cz>
- Add sanlock-lto-disable-fpie.patch in order to handle
  boo#1138836. Extra -fPIE is stripped.
* Mon Jun 17 2019 Jim Fehlig <jfehlig@suse.com>
- Update to sanlock 3.8.0
  - Add support for python 3
  - Add support for 4k sector size
  - More API test coverage
  - Misc bug fixes and improvements
  - Dropped patches:
    revert-5d535c0d-py2.patch, sanlock-python3.patch
* Tue Apr  9 2019 Jim Fehlig <jfehlig@suse.com>
- Update to sanlock 3.7.1
  - Fix bugs in read_resource_owners
* Thu Mar 21 2019 Jim Fehlig <jfehlig@suse.com>
- Update to sanlock 3.7.0
  - Allow setting max_sectors_kb
  - Configurable sector size and align size
  - Support SANLOCK_RUN_DIR and SANLOCK_PRIVILEGED environment
    variables
  - Revert upstream commit forcing use of python2
    revert-5d535c0d-py2.patch
  - Dropped patches:
    sanlock-old_blkid.patch, suse-fix-link-errors.patch
* Mon Mar 26 2018 jfehlig@suse.com
- Fix module initialization to work with both python2 and python3
  Updated sanlock-python3.patch
  bsc#1076414
* Wed Mar 21 2018 tchvatal@suse.com
- Add code to build with both python2 and python3 bindings with
  provided condition so we can later on optionalize python2 builds
  sanlock-python3.patch
  bsc#1076414
* Wed Mar 21 2018 tchvatal@suse.com
- Cleanup with spec-cleaner (only support SLE12 and newer)
- Add -fno-strict-aliasing to python subpkg
- Create rcsanlk-resetd link for the service control
* Tue Dec  5 2017 jfehlig@suse.com
- Update to sanlock 3.6.0
  - preserve dblock values when setting shared flag
  - fix detection of shared lease
  - fix release interference with paxos
  - retry ballot when new lver is seen
  - add a NOWAIT flag for convert
* Thu Nov 23 2017 rbrown@suse.com
- Replace references to /var/adm/fillup-templates with new
  %%_fillupdir macro (boo#1069468)
* Wed Jun  7 2017 jfehlig@suse.com
- Update to sanlock 3.5.0
  - increase open file limit to 2048
  - add option to log UTC timestamps
  - doc improvements
- Remove support for old, non-systemd distros
* Fri Apr  7 2017 jengelh@inai.de
- Remove redundant %%if guards around %%package and scriptlets.
  They are only really needed for %%files.
- Replace $RPM_ shell vars by modern macros.
- Trim descriptions a little.
* Mon Apr  3 2017 jfehlig@suse.com
- spec: fix permissions on config files
* Mon Apr  3 2017 jfehlig@suse.com
- Use System/Base in fence and reset subpackages
* Mon Apr  3 2017 mpluskal@suse.com
- Use correct permissions on unit files (bsc#1032046)
* Wed Mar 29 2017 meissner@suse.com
- prerequire groupadd and useradd.
* Wed Mar 15 2017 zren@suse.com
- Update to sanlock 3.4.0
- Enable fence_sanlockd so that lvm2 can use the new lvmlocked
  feature in cluster environment
  * fix unpackaged file - /usr/sbin/rcfence_sanlockd
  * refresh suse-no-date-time.patch to cover the same issue
    in fence_sanlockd
  * adjust fence_sanlockd.init and suse-systemd.patch
- Enable sanlk-reset subpackage
- Fix "undefined symbol to io_suspend" at runtime, bsc#1030060
  suse-fix-link-errors.patch
* Sat Mar 11 2017 kukuk@suse.com
- Add group disk as Prerequires
* Fri Jun 19 2015 jfehlig@suse.com
- Update to sanlock 3.2.4
  - Add conf file sanlock.conf
  - Add pkgconfig files
  - Numerous improvements and bug fixes.  For details, see
    https://git.fedorahosted.org/cgit/sanlock.git/log/
* Sat Mar 21 2015 p.drouand@gmail.com
- Update to version 3.2.2
  * reset: native timeout
  * reset: Reduce logging noise to stderr and syslog
  * sanlock, reset: coverity fixes
- Refactor specfile to be policy compliant
- Use download Url as source
- Add xz build requires; needed as the source tarball is compressed
  with xz
- Do not provide both systemd and sysvinit support on a same system;
  it's redundant
- Add systemd backward compatibility links
- Update patchs:
  * suse-no-date-time.patch
  * suse-systemd.patch
* Fri Jan 10 2014 jfehlig@suse.com
- Update to sanlock 3.1.0
  - fix debug status for add/rem lockspace
  - add_lockspace error for conflicting lockspace definitions
  - transient timeout handling in acquire and release
  - allow more than 8 resources per client
  - fix spaces in path names
* Thu Aug  1 2013 jfehlig@suse.com
- Update to sanlock 3.0.1
  - Add lvb feature
  - Various bug fixes and improvements
  - Drop upstream patch 0001-fix-systemd-service-files.patch
* Fri Mar  1 2013 coolo@suse.com
- update license to new format
* Mon Dec  3 2012 jfehlig@suse.com
- Update to sanlock 2.6
  - Add fence_sanlock, a fence agent that uses /dev/watchdog to
    reset hosts (currently disabled)
  - Various bug fixes and improvements
* Tue Mar 13 2012 jfehlig@suse.com
- Update to sanlock 2.1
  - python: release leases for other processes
  - python: add shared resource parameter to acquire
  - add a logrotate file
  - status for all shared tokens
  - retry transient sh failures
  - fix inquire state string
* Wed Jan 25 2012 jfehlig@suse.com
- Fixed init script packaging
* Mon Jan 23 2012 jfehlig@suse.com
- Fixed specification of licenses
* Mon Jan 23 2012 jfehlig@suse.com
- Source sanlock tarball instead of download URL
* Fri Jan 20 2012 jfehlig@suse.com
- Don't use __DATE__ and __TIME__ macros when printing version
- Don't add user 'sanlock' for now
* Fri Jan 20 2012 jfehlig@suse.com
- Fix some rpmlint warnings
* Fri Jan 20 2012 jfehlig@suse.com
- Add support for systemd
- Add /etc/sysconfig/sanlock configuration file
* Thu Jan 19 2012 jfehlig@suse.com
- Pass '-g sanlock' option to wdmd
* Wed Jan 18 2012 jfehlig@suse.com
- Update to sanlock 1.9
- Add SUSE init script for sanlock and wdmd
* Thu Dec 22 2011 ro@suse.de
- initial package based on package from openstack
